/*
 * Copyright (c) 2002-2013, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.document.service.spaces;

import fr.paris.lutece.plugins.document.business.spaces.DocumentSpace;
import fr.paris.lutece.plugins.document.business.spaces.DocumentSpaceHome;
import fr.paris.lutece.plugins.document.service.DocumentPlugin;
import fr.paris.lutece.plugins.document.utils.IntegerUtils;
import fr.paris.lutece.portal.service.rbac.Permission;
import fr.paris.lutece.portal.service.rbac.ResourceIdService;
import fr.paris.lutece.portal.service.rbac.ResourceType;
import fr.paris.lutece.portal.service.rbac.ResourceTypeManager;
import fr.paris.lutece.portal.service.util.AppLogService;
import fr.paris.lutece.util.ReferenceList;

import java.util.Locale;

import org.apache.commons.lang.StringUtils;


/**
 * Resource Id service for RBAC features to control access to document spaces
 */
public class SpaceResourceIdService extends ResourceIdService
{
    /** Permission for creating a space */
    public static final String PERMISSION_CREATE = "CREATE";

    /** Permission for viewing a space */
    public static final String PERMISSION_VIEW = "VIEW";

    /** Permission for deleting a space */
    public static final String PERMISSION_DELETE = "DELETE";

    /** Permission for modifying a space */
    public static final String PERMISSION_MODIFY = "MODIFY";

    /** Permission for moving a space */
    public static final String PERMISSION_MOVE = "MOVE";
    private static final String PROPERTY_LABEL_RESOURCE_TYPE = "document.spaces.resourceType";
    private static final String PROPERTY_LABEL_CREATE = "document.spaces.permission.label.create";
    private static final String PROPERTY_LABEL_VIEW = "document.spaces.permission.label.view";
    private static final String PROPERTY_LABEL_DELETE = "document.spaces.permission.label.delete";
    private static final String PROPERTY_LABEL_MODIFY = "document.spaces.permission.label.modify";
    private static final String PROPERTY_LABEL_MOVE = "document.spaces.permission.label.move";

    /** Creates a new instance of SpaceResourceIdService */
    public SpaceResourceIdService(  )
    {
        setPluginName( DocumentPlugin.PLUGIN_NAME );
    }

    /**
     * Initializes the service
     */
    public void register(  )
    {
        ResourceType rt = new ResourceType(  );
        rt.setResourceIdServiceClass( SpaceResourceIdService.class.getName(  ) );
        rt.setPluginName( DocumentPlugin.PLUGIN_NAME );
        rt.setResourceTypeKey( DocumentSpace.RESOURCE_TYPE );
        rt.setResourceTypeLabelKey( PROPERTY_LABEL_RESOURCE_TYPE );

        Permission p = new Permission(  );
        p.setPermissionKey( PERMISSION_CREATE );
        p.setPermissionTitleKey( PROPERTY_LABEL_CREATE );
        rt.registerPermission( p );

        p = new Permission(  );
        p.setPermissionKey( PERMISSION_VIEW );
        p.setPermissionTitleKey( PROPERTY_LABEL_VIEW );
        rt.registerPermission( p );

        p = new Permission(  );
        p.setPermissionKey( PERMISSION_DELETE );
        p.setPermissionTitleKey( PROPERTY_LABEL_DELETE );
        rt.registerPermission( p );

        p = new Permission(  );
        p.setPermissionKey( PERMISSION_MODIFY );
        p.setPermissionTitleKey( PROPERTY_LABEL_MODIFY );
        rt.registerPermission( p );

        p = new Permission(  );
        p.setPermissionKey( PERMISSION_MOVE );
        p.setPermissionTitleKey( PROPERTY_LABEL_MOVE );
        rt.registerPermission( p );

        ResourceTypeManager.registerResourceType( rt );
    }

    /**
     * Returns a list of resource ids
     * @param locale The current locale
     * @return A list of resource ids
     */
    public ReferenceList getResourceIdList( Locale locale )
    {
        return DocumentSpaceHome.getDocumentSpaceList(  );
    }

    /**
     * Returns the Title of a given resource
     * @param strId The Id of the resource
     * @param locale The current locale
     * @return The Title of a given resource
     */
    public String getTitle( String strId, Locale locale )
    {
        if ( IntegerUtils.isNumeric( strId ) )
        {
            int nSpaceId = IntegerUtils.convert( strId );
            DocumentSpace space = DocumentSpaceHome.findByPrimaryKey( nSpaceId );

            if ( space != null )
            {
                return space.getName(  );
            }

            AppLogService.error( "SpaceResourceIdService - The space is not found." );
        }
        else
        {
            AppLogService.error( "SpaceResourceIdService - The id is blank or is not numeric." );
        }

        return StringUtils.EMPTY;
    }
}
